try:
    import cv2
    import numpy as np
except:
    cv2 = None

east_filename = "frozen_east_text_detection.pb"
def getTextDector():
    pass

def findTextBoxes(image):
    (H, W) = image.shape[:2]

    # set the new width and height and then determine the ratio in change
    # for both the width and height
    (newW, newH) = (args["width"], args["height"])
    rW = W / float(newW)
    rH = H / float(newH)

    # resize the image and grab the new image dimensions
    image = cv2.resize(image, (newW, newH))
    (H, W) = image.shape[:2]

    # define the two output layer names for the EAST detector model that
    # we are interested -- the first is the output probabilities and the
    # second can be used to derive the bounding box coordinates of text
    layerNames = [
        "feature_fusion/Conv_7/Sigmoid",
        "feature_fusion/concat_3"]

    # load the pre-trained EAST text detector
    print("[INFO] loading EAST text detector...")
    net = cv2.dnn.readNet(args["east"])

    # construct a blob from the image and then perform a forward pass of
    # the model to obtain the two output layer sets
    blob = cv2.dnn.blobFromImage(image, 1.0, (W, H),
        (123.68, 116.78, 103.94), swapRB=True, crop=False)
    start = time.time()
    net.setInput(blob)
    (scores, geometry) = net.forward(layerNames)
    end = time.time()

    # show timing information on text prediction
    print(("[INFO] text detection took {:.6f} seconds".format(end - start)))

    # grab the number of rows and columns from the scores volume, then
    # initialize our set of bounding box rectangles and corresponding
    # confidence scores
    (numRows, numCols) = scores.shape[2:4]
    rects = []
    confidences = []

    # loop over the number of rows
    for y in range(0, numRows):
        # extract the scores (probabilities), followed by the geometrical
        # data used to derive potential bounding box coordinates that
        # surround text
        scoresData = scores[0, 0, y]
        xData0 = geometry[0, 0, y]
        xData1 = geometry[0, 1, y]
        xData2 = geometry[0, 2, y]
        xData3 = geometry[0, 3, y]
        anglesData = geometry[0, 4, y]

        # loop over the number of columns
        for x in range(0, numCols):
            # if our score does not have sufficient probability, ignore it
            if scoresData[x] < args["min_confidence"]:
                continue

            # compute the offset factor as our resulting feature maps will
            # be 4x smaller than the input image
            (offsetX, offsetY) = (x * 4.0, y * 4.0)

            # extract the rotation angle for the prediction and then
            # compute the sin and cosine
            angle = anglesData[x]
            cos = np.cos(angle)
            sin = np.sin(angle)

            # use the geometry volume to derive the width and height of
            # the bounding box
            h = xData0[x] + xData2[x]
            w = xData1[x] + xData3[x]

            # compute both the starting and ending (x, y)-coordinates for
            # the text prediction bounding box
            endX = int(offsetX + (cos * xData1[x]) + (sin * xData2[x]))
            endY = int(offsetY - (sin * xData1[x]) + (cos * xData2[x]))
            startX = int(endX - w)
            startY = int(endY - h)

            # add the bounding box coordinates and probability score to
            # our respective lists
            rects.append((startX, startY, endX, endY))
            confidences.append(scoresData[x])

    # apply non-maxima suppression to suppress weak, overlapping bounding
    # boxes
    boxes = non_max_suppression(np.array(rects), probs=confidences)



def non_max_suppression(boxes, probs=None, overlapThresh=0.3):
    # if there are no boxes, return an empty list
    if len(boxes) == 0:
        return []

    # if the bounding boxes are integers, convert them to floats -- this
    # is important since we'll be doing a bunch of divisions
    if boxes.dtype.kind == "i":
        boxes = boxes.astype("float")

    # initialize the list of picked indexes
    pick = []

    # grab the coordinates of the bounding boxes
    x1 = boxes[:, 0]
    y1 = boxes[:, 1]
    x2 = boxes[:, 2]
    y2 = boxes[:, 3]

    # compute the area of the bounding boxes and grab the indexes to sort
    # (in the case that no probabilities are provided, simply sort on the
    # bottom-left y-coordinate)
    area = (x2 - x1 + 1) * (y2 - y1 + 1)
    idxs = y2

    # if probabilities are provided, sort on them instead
    if probs is not None:
        idxs = probs

    # sort the indexes
    idxs = np.argsort(idxs)

    # keep looping while some indexes still remain in the indexes list
    while len(idxs) > 0:
        # grab the last index in the indexes list and add the index value
        # to the list of picked indexes
        last = len(idxs) - 1
        i = idxs[last]
        pick.append(i)

        # find the largest (x, y) coordinates for the start of the bounding
        # box and the smallest (x, y) coordinates for the end of the bounding
        # box
        xx1 = np.maximum(x1[i], x1[idxs[:last]])
        yy1 = np.maximum(y1[i], y1[idxs[:last]])
        xx2 = np.minimum(x2[i], x2[idxs[:last]])
        yy2 = np.minimum(y2[i], y2[idxs[:last]])

        # compute the width and height of the bounding box
        w = np.maximum(0, xx2 - xx1 + 1)
        h = np.maximum(0, yy2 - yy1 + 1)

        # compute the ratio of overlap
        overlap = (w * h) / area[idxs[:last]]

        # delete all indexes from the index list that have overlap greater
        # than the provided overlap threshold
        idxs = np.delete(idxs, np.concatenate(([last],
            np.where(overlap > overlapThresh)[0])))

    # return only the bounding boxes that were picked
    return boxes[pick].astype("int")


# Generated by Django 4.1.5 on 2024-06-07 20:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0020_cloudcredentials_cloudcredentialsmetadata'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cloudcredentials',
            name='group',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cloud_credentials', to='account.group'),
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-13 02:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wiki', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pagemedia',
            name='entry',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='media_library', to='wiki.page'),
        ),
    ]



import requests
from objict import objict
from rest import __version__ as restit_version
from urllib.parse import urlparse, parse_qs, urlunparse
from rest import helpers as rh

def getURL(host, path):
    if path is None:
        return host
    if host.endswith("/"):
        host = host[:-1]
    if path.startswith("/"):
        path = path[1:]
    if host.startswith("http"):
        return "{host}/{path}"
    return f"https://{host}/{path}" 


def REQUEST(method, host, path=None, data=None, params=None, 
            headers=None, files=None, session=None, post_json=False,
            timeout=None, verify=True, raw_response=False):
    url = getURL(host, path)
    if headers is None:
        headers = objict()

    fields = dict(url=url, params=params, files=files, verify=verify, timeout=timeout)
    if isinstance(data, dict):
        post_json = True
        fields["json"] = objict.fromdict(data).toJSON()
    elif data is not None:
        fields["data"] = data

    # headers["Accept"] = 'application/json'
    if post_json:
        headers['Content-type'] = 'application/json'
    if "User-Agent" not in headers:
        headers["User-Agent"] = f"restit/{restit_version}"
    fields["headers"] = headers
    if session is None:
        session = requests

    rh.debug("net.REQUEST", fields)
    if raw_response:
        return getattr(session, method.lower())(**fields)

    try:
        res = getattr(session, method.lower())(**fields)
    except Exception as err:
        return objict(status=False, status_code=500, error=str(err))
    return processResponse(res, url)


def processResponse(res, url):
    data = res.text
    try:
        data = objict.fromdict(res.json())
    except Exception as err:
        return objict(status=False, status_code=res.status_code, data=res.text, error=str(err))

    if res.status_code not in [200, 201]:
        if isinstance(data, dict) and data.error:
            data.status = False
            data.status_code = res.status_code
            return data
        return objict(status=False, status_code=res.status_code, error=res.text)
    if data.data:
        data = data.data
    return objict(status=True, status_code=res.status_code, data=data)


def parse_params(url):
    """
    Extracts the base URL (without parameters) and parameters from a URL.

    Parameters:
    - url (str): The URL from which to extract the base URL and parameters.

    Returns:
    - tuple: A tuple containing the base URL (str) and a dictionary of parameters.
    """
    # Parse the URL
    parsed_url = urlparse(url)
    # Extract and convert the query part to a dictionary
    params = parse_qs(parsed_url.query)
    # Convert the values lists to single values if they contain only one item
    params_single_value = {k: v[0] if len(v) == 1 else v for k, v in params.items()}
    # Reconstruct the URL without the query part
    base_url = urlunparse(parsed_url._replace(query=""))
    return base_url, params_single_value



# Copyright 2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Multi-credential file store with lock support.

This module implements a JSON credential store where multiple
credentials can be stored in one file. That file supports locking
both in a single process and across processes.

The credential themselves are keyed off of:
* client_id
* user_agent
* scope

The format of the stored data is like so:
{
  'file_version': 1,
  'data': [
    {
      'key': {
        'clientId': '<client id>',
        'userAgent': '<user agent>',
        'scope': '<scope>'
      },
      'credential': {
        # JSON serialized Credentials.
      }
    }
  ]
}
"""

__author__ = 'jbeda@google.com (Joe Beda)'

import base64
import errno
import logging
import os
import threading

from .anyjson import simplejson
from oauth2client.client import Storage as BaseStorage
from oauth2client.client import Credentials
from oauth2client import util
from .locked_file import LockedFile

logger = logging.getLogger(__name__)

# A dict from 'filename'->_MultiStore instances
_multistores = {}
_multistores_lock = threading.Lock()


class Error(Exception):
  """Base error for this module."""
  pass


class NewerCredentialStoreError(Error):
  """The credential store is a newer version that supported."""
  pass


@util.positional(4)
def get_credential_storage(filename, client_id, user_agent, scope,
                           warn_on_readonly=True):
  """Get a Storage instance for a credential.

  Args:
    filename: The JSON file storing a set of credentials
    client_id: The client_id for the credential
    user_agent: The user agent for the credential
    scope: string or iterable of strings, Scope(s) being requested
    warn_on_readonly: if True, log a warning if the store is readonly

  Returns:
    An object derived from client.Storage for getting/setting the
    credential.
  """
  # Recreate the legacy key with these specific parameters
  key = {'clientId': client_id, 'userAgent': user_agent,
         'scope': util.scopes_to_string(scope)}
  return get_credential_storage_custom_key(
      filename, key, warn_on_readonly=warn_on_readonly)


@util.positional(2)
def get_credential_storage_custom_string_key(
    filename, key_string, warn_on_readonly=True):
  """Get a Storage instance for a credential using a single string as a key.

  Allows you to provide a string as a custom key that will be used for
  credential storage and retrieval.

  Args:
    filename: The JSON file storing a set of credentials
    key_string: A string to use as the key for storing this credential.
    warn_on_readonly: if True, log a warning if the store is readonly

  Returns:
    An object derived from client.Storage for getting/setting the
    credential.
  """
  # Create a key dictionary that can be used
  key_dict = {'key': key_string}
  return get_credential_storage_custom_key(
      filename, key_dict, warn_on_readonly=warn_on_readonly)


@util.positional(2)
def get_credential_storage_custom_key(
    filename, key_dict, warn_on_readonly=True):
  """Get a Storage instance for a credential using a dictionary as a key.

  Allows you to provide a dictionary as a custom key that will be used for
  credential storage and retrieval.

  Args:
    filename: The JSON file storing a set of credentials
    key_dict: A dictionary to use as the key for storing this credential. There
      is no ordering of the keys in the dictionary. Logically equivalent
      dictionaries will produce equivalent storage keys.
    warn_on_readonly: if True, log a warning if the store is readonly

  Returns:
    An object derived from client.Storage for getting/setting the
    credential.
  """
  multistore = _get_multistore(filename, warn_on_readonly=warn_on_readonly)
  key = util.dict_to_tuple_key(key_dict)
  return multistore._get_storage(key)


@util.positional(1)
def get_all_credential_keys(filename, warn_on_readonly=True):
  """Gets all the registered credential keys in the given Multistore.

  Args:
    filename: The JSON file storing a set of credentials
    warn_on_readonly: if True, log a warning if the store is readonly

  Returns:
    A list of the credential keys present in the file.  They are returned as
    dictionaries that can be passed into get_credential_storage_custom_key to
    get the actual credentials.
  """
  multistore = _get_multistore(filename, warn_on_readonly=warn_on_readonly)
  multistore._lock()
  try:
    return multistore._get_all_credential_keys()
  finally:
    multistore._unlock()


@util.positional(1)
def _get_multistore(filename, warn_on_readonly=True):
  """A helper method to initialize the multistore with proper locking.

  Args:
    filename: The JSON file storing a set of credentials
    warn_on_readonly: if True, log a warning if the store is readonly

  Returns:
    A multistore object
  """
  filename = os.path.expanduser(filename)
  _multistores_lock.acquire()
  try:
    multistore = _multistores.setdefault(
        filename, _MultiStore(filename, warn_on_readonly=warn_on_readonly))
  finally:
    _multistores_lock.release()
  return multistore


class _MultiStore(object):
  """A file backed store for multiple credentials."""

  @util.positional(2)
  def __init__(self, filename, warn_on_readonly=True):
    """Initialize the class.

    This will create the file if necessary.
    """
    self._file = LockedFile(filename, 'r+b', 'rb')
    self._thread_lock = threading.Lock()
    self._read_only = False
    self._warn_on_readonly = warn_on_readonly

    self._create_file_if_needed()

    # Cache of deserialized store. This is only valid after the
    # _MultiStore is locked or _refresh_data_cache is called. This is
    # of the form of:
    #
    # ((key, value), (key, value)...) -> OAuth2Credential
    #
    # If this is None, then the store hasn't been read yet.
    self._data = None

  class _Storage(BaseStorage):
    """A Storage object that knows how to read/write a single credential."""

    def __init__(self, multistore, key):
      self._multistore = multistore
      self._key = key

    def acquire_lock(self):
      """Acquires any lock necessary to access this Storage.

      This lock is not reentrant.
      """
      self._multistore._lock()

    def release_lock(self):
      """Release the Storage lock.

      Trying to release a lock that isn't held will result in a
      RuntimeError.
      """
      self._multistore._unlock()

    def locked_get(self):
      """Retrieve credential.

      The Storage lock must be held when this is called.

      Returns:
        oauth2client.client.Credentials
      """
      credential = self._multistore._get_credential(self._key)
      if credential:
        credential.set_store(self)
      return credential

    def locked_put(self, credentials):
      """Write a credential.

      The Storage lock must be held when this is called.

      Args:
        credentials: Credentials, the credentials to store.
      """
      self._multistore._update_credential(self._key, credentials)

    def locked_delete(self):
      """Delete a credential.

      The Storage lock must be held when this is called.

      Args:
        credentials: Credentials, the credentials to store.
      """
      self._multistore._delete_credential(self._key)

  def _create_file_if_needed(self):
    """Create an empty file if necessary.

    This method will not initialize the file. Instead it implements a
    simple version of "touch" to ensure the file has been created.
    """
    if not os.path.exists(self._file.filename()):
      old_umask = os.umask(0o177)
      try:
        open(self._file.filename(), 'a+b').close()
      finally:
        os.umask(old_umask)

  def _lock(self):
    """Lock the entire multistore."""
    self._thread_lock.acquire()
    self._file.open_and_lock()
    if not self._file.is_locked():
      self._read_only = True
      if self._warn_on_readonly:
        logger.warn('The credentials file (%s) is not writable. Opening in '
                    'read-only mode. Any refreshed credentials will only be '
                    'valid for this run.' % self._file.filename())
    if os.path.getsize(self._file.filename()) == 0:
      logger.debug('Initializing empty multistore file')
      # The multistore is empty so write out an empty file.
      self._data = {}
      self._write()
    elif not self._read_only or self._data is None:
      # Only refresh the data if we are read/write or we haven't
      # cached the data yet. If we are readonly, we assume is isn't
      # changing out from under us and that we only have to read it
      # once. This prevents us from whacking any new access keys that
      # we have cached in memory but were unable to write out.
      self._refresh_data_cache()

  def _unlock(self):
    """Release the lock on the multistore."""
    self._file.unlock_and_close()
    self._thread_lock.release()

  def _locked_json_read(self):
    """Get the raw content of the multistore file.

    The multistore must be locked when this is called.

    Returns:
      The contents of the multistore decoded as JSON.
    """
    assert self._thread_lock.locked()
    self._file.file_handle().seek(0)
    return simplejson.load(self._file.file_handle())

  def _locked_json_write(self, data):
    """Write a JSON serializable data structure to the multistore.

    The multistore must be locked when this is called.

    Args:
      data: The data to be serialized and written.
    """
    assert self._thread_lock.locked()
    if self._read_only:
      return
    self._file.file_handle().seek(0)
    simplejson.dump(data, self._file.file_handle(), sort_keys=True, indent=2)
    self._file.file_handle().truncate()

  def _refresh_data_cache(self):
    """Refresh the contents of the multistore.

    The multistore must be locked when this is called.

    Raises:
      NewerCredentialStoreError: Raised when a newer client has written the
        store.
    """
    self._data = {}
    try:
      raw_data = self._locked_json_read()
    except Exception:
      logger.warn('Credential data store could not be loaded. '
                  'Will ignore and overwrite.')
      return

    version = 0
    try:
      version = raw_data['file_version']
    except Exception:
      logger.warn('Missing version for credential data store. It may be '
                  'corrupt or an old version. Overwriting.')
    if version > 1:
      raise NewerCredentialStoreError(
          'Credential file has file_version of %d. '
          'Only file_version of 1 is supported.' % version)

    credentials = []
    try:
      credentials = raw_data['data']
    except (TypeError, KeyError):
      pass

    for cred_entry in credentials:
      try:
        (key, credential) = self._decode_credential_from_json(cred_entry)
        self._data[key] = credential
      except:
        # If something goes wrong loading a credential, just ignore it
        logger.info('Error decoding credential, skipping', exc_info=True)

  def _decode_credential_from_json(self, cred_entry):
    """Load a credential from our JSON serialization.

    Args:
      cred_entry: A dict entry from the data member of our format

    Returns:
      (key, cred) where the key is the key tuple and the cred is the
        OAuth2Credential object.
    """
    raw_key = cred_entry['key']
    key = util.dict_to_tuple_key(raw_key)
    credential = None
    credential = Credentials.new_from_json(simplejson.dumps(cred_entry['credential']))
    return (key, credential)

  def _write(self):
    """Write the cached data back out.

    The multistore must be locked.
    """
    raw_data = {'file_version': 1}
    raw_creds = []
    raw_data['data'] = raw_creds
    for (cred_key, cred) in list(self._data.items()):
      raw_key = dict(cred_key)
      raw_cred = simplejson.loads(cred.to_json())
      raw_creds.append({'key': raw_key, 'credential': raw_cred})
    self._locked_json_write(raw_data)

  def _get_all_credential_keys(self):
    """Gets all the registered credential keys in the multistore.

    Returns:
      A list of dictionaries corresponding to all the keys currently registered
    """
    return [dict(key) for key in list(self._data.keys())]

  def _get_credential(self, key):
    """Get a credential from the multistore.

    The multistore must be locked.

    Args:
      key: The key used to retrieve the credential

    Returns:
      The credential specified or None if not present
    """
    return self._data.get(key, None)

  def _update_credential(self, key, cred):
    """Update a credential and write the multistore.

    This must be called when the multistore is locked.

    Args:
      key: The key used to retrieve the credential
      cred: The OAuth2Credential to update/set
    """
    self._data[key] = cred
    self._write()

  def _delete_credential(self, key):
    """Delete a credential and write the multistore.

    This must be called when the multistore is locked.

    Args:
      key: The key used to retrieve the credential
    """
    try:
      del self._data[key]
    except KeyError:
      pass
    self._write()

  def _get_storage(self, key):
    """Get a Storage object to get/set a credential.

    This Storage is a 'view' into the multistore.

    Args:
      key: The key used to retrieve the credential

    Returns:
      A Storage object that can be used to get/set this cred
    """
    return self._Storage(self, key)



from rest import settings
from rest.log import getLogger
from objict import objict
import metrics
import boto3
import threading
from . import render as mr

EMAIL_LOGGER = getLogger("email", filename="email.log")
# ses settings
SES_ACCESS_KEY = settings.SES_ACCESS_KEY
SES_SECRET_KEY = settings.SES_SECRET_KEY
SES_REGION = settings.SES_REGION
EMAIL_METRICS = settings.EMAIL_METRICS
EMAIL_ASYNC_AS_TASK = settings.EMAIL_ASYNC_AS_TASK

# write python regect to detect if text is html


def send(sender, recipients, subject, message, attachments=None, replyto=None, fail_silently=False, do_async=False):
    html = None
    text = None
    if mr.isHTML(message):
        # assume html?
        html = message
    else:
        text = message
    msg = mr.createMessage(sender, recipients, subject, text, html, attachments=attachments, replyto=replyto)
    # METRICS FAIL WHEN SWITCHING THREADS
    # if do_async:
    #     t = threading.Thread(target=sendMail, args=[msg.msg, msg.sender, msg.recipients])
    #     t.start()
    #     return True
    return sendMail(msg.msg, msg.sender, msg.recipients, fail_silently=fail_silently)


def sendMail(msg, sender, recipients, fail_silently=True):
    return sendMailViaSES(msg, sender, recipients, fail_silently)


def sendMailViaSES(msg, sender, recipients, fail_silently=True):
    try:
        ses_client = getSES(SES_ACCESS_KEY, SES_SECRET_KEY, SES_REGION)
        ses_client.send_raw_email(
            Source=sender,
            Destinations=recipients,
            RawMessage={'Data': msg.as_string()}
        )
        if EMAIL_METRICS:
            metrics.metric("emails_sent", category="email", min_granularity="hourly")
        return True
    except Exception as err:
        if EMAIL_METRICS:
            metrics.metric("email_errors", category="email", min_granularity="hourly")
        EMAIL_LOGGER.exception(err)
        EMAIL_LOGGER.error(msg.as_string())
        if not fail_silently:
            raise err
    return False


def getSES(access_key, secret_key, region):
    return boto3.client(
        'ses',
        aws_access_key_id=access_key,
        aws_secret_access_key=secret_key,
        region_name=region)



# Generated by Django 4.1.3 on 2022-12-14 18:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('location', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='geoip',
            name='subnet',
            field=models.CharField(db_index=True, default=None, max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name='geoip',
            name='ip',
            field=models.CharField(db_index=True, max_length=64),
        ),
    ]



import jwt
from objict import objict
import time
import uuid
from rest import settings

JWT_EXP_6_HOURS = 21600
JWT_EXP_12_HOURS = 43200
JWT_EXP_1_DAY = 86400
JWT_EXP_2_DAY = 172800
JWT_EXP_7_DAY = 604800
JWT_EXP_30_MIN = 1800
JWT_EXP_15_MIN = 900
JWT_KEY = settings.SECRET_KEY
JWT_EXPIRES = settings.get("JWT_EXPIRES", JWT_EXP_30_MIN)
JWT_REFRESH_EXPIRES = settings.get("JWT_REFRESH_EXPIRES", JWT_EXP_2_DAY)


class JWToken():
    def __init__(self, token=None, user_id=None, key=JWT_KEY, 
                 device_id=None, alg="HS256", access_expires_in=JWT_EXPIRES,
                 refresh_expires_in=JWT_REFRESH_EXPIRES):
        # takes full JWT token header.payload.signature
        self.token = token
        self.key = key
        self.alg = alg
        self.access_expires_in = access_expires_in
        self.refresh_expires_in = refresh_expires_in
        self._is_valid = None
        if token is None:
            self.payload = objict(token_type="access", jti=uuid.uuid4().hex)
            if user_id is not None:
                self.payload["user_id"] = user_id
            if device_id is not None:
                self.payload["device_id"] = device_id
            self.refresh()
        else:
            self.payload = objict.fromdict(jwt.decode(token, key, algorithms=alg, options=dict(verify_signature=False)))
            
    @property
    def is_valid(self):
        if self._is_valid is not None:
            return self._is_valid
        try:
            objict.fromdict(jwt.decode(self.token, self.key, algorithms=self.alg))
            self._is_valid = True
        except Exception:
            self._is_valid = False
        return self._is_valid

    @property
    def session_id(self):
        if self.payload:
            return self.payload.jti
        return None

    def isExpired(self):
        if self.payload.exp is None:
            return False
        return self.payload.exp > time.time()

    def refresh(self, expires_in=None):
        self.payload["token_type"] = "access"
        self.payload["iat"] = int(time.time())
        if expires_in is None:
            expires_in = self.access_expires_in
        self.payload["exp"] = int(time.time()) + expires_in

    @property
    def access_token(self):
        return jwt.encode(self.payload, self.key, algorithm=self.alg)

    @property
    def refresh_token(self):
        r_payload = objict.fromdict(self.payload)
        r_payload["token_type"] = "refresh"
        r_payload["exp"] = int(time.time()) + self.refresh_expires_in
        return jwt.encode(r_payload, self.key)





# Generated by Django 4.1.3 on 2022-12-20 04:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('account', '0012_settings_settingsmetadata'),
    ]

    operations = [
        migrations.CreateModel(
            name='Metrics',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('start', models.DateTimeField(db_index=True)),
                ('granularity', models.CharField(db_index=True, max_length=64)),
                ('uuid', models.SlugField(max_length=124, unique=True)),
                ('slug', models.SlugField(max_length=124)),
                ('k1', models.SlugField(default=None, max_length=64, null=True)),
                ('v1', models.IntegerField(default=0)),
                ('k2', models.SlugField(default=None, max_length=64, null=True)),
                ('v2', models.IntegerField(default=0)),
                ('k3', models.SlugField(default=None, max_length=64, null=True)),
                ('v3', models.IntegerField(default=0)),
                ('k4', models.SlugField(default=None, max_length=64, null=True)),
                ('v4', models.IntegerField(default=0)),
                ('k5', models.SlugField(default=None, max_length=64, null=True)),
                ('v5', models.IntegerField(default=0)),
                ('k6', models.SlugField(default=None, max_length=64, null=True)),
                ('v6', models.IntegerField(default=0)),
                ('k7', models.SlugField(default=None, max_length=64, null=True)),
                ('v7', models.IntegerField(default=0)),
                ('k8', models.SlugField(default=None, max_length=64, null=True)),
                ('v8', models.IntegerField(default=0)),
                ('k9', models.SlugField(default=None, max_length=64, null=True)),
                ('v9', models.IntegerField(default=0)),
                ('k10', models.SlugField(default=None, max_length=64, null=True)),
                ('v10', models.IntegerField(default=0)),
                ('group', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='account.group')),
            ],
        ),
    ]



from rest import decorators as rd
from rest import views as rv
from rest import helpers as rh
from account.models import Group, Membership, Member, GroupFeed, MemberFeed


@rd.url(r'^group$')
@rd.url(r'^group/$')  # required for legacy support
@rd.url(r'^group/(?P<pk>\d+)$')
@rd.login_required
def rest_on_group(request, pk=None):
    return Group.on_rest_request(request, pk)


@rd.urlGET(r'^member/groups$')
@rd.login_required
def member_groups(request):
    member_id = request.DATA.get(["member", "member_id"], request.member.id)
    if not member_id:
        return rv.restPermissionDenied(request)
    member = Member.objects.filter(pk=member_id).last()
    return Group.on_rest_list(request, member.getGroups())


@rd.url(r'^group/invite$')
@rd.url(r'^group/invite/(?P<group_id>\d+)$')
@rd.perm_required(["manage_members", "invite_members", "manage_groups"])
def rest_on_group_invite(request, group_id=None):
    if group_id:
        group = Group.objects.filter(pk=group_id).first()
        if not group:
            return rv.restPermissionDenied(request)
    else:
        group = request.group
    if group is None:
        return rv.restPermissionDenied(request, "requires group")
    # this will throw exception on failure
    ms = None
    member = None
    member_id = request.DATA.get(["member", "member_id"])
    if member_id:
        member = Member.objects.filter(pk=member_id).last()
    if member is None:
        # create new member, but only allow basic information
        data = request.DATA.fromKeys(["username", "email", "phone_number", "display_name", "first_name", "last_name"])
        if not data or not data.email:
            return rv.restPermissionDenied(request, "missing fields")
        if data.email:
            member = Member.GetMember(data.email.lower().strip())
        elif data.username:
            member = Member.GetMember(data.username.lower().strip())
        elif data.phone_number:
            member = Member.GetMemberByPhone(data.phone_number)
        if member is None:
            member = Member.createFromDict(request, data)
        member.auditLog(f"created by {request.member.username}", "created", level=21)
    
    ms = Membership.objects.filter(member=member, group=group).first()
    if ms is None:
        member.auditLog(f"added to {group.name}:{group.id} by {request.member.username}", "group_invite", level=21)
        ms = group.invite(member, request.DATA.get("role", "guest"))
    elif not ms.is_enabled:
        # re-enable the member if they are being re-invited after being disabled
        if not ms.member.is_active:
            # we need to make sure we only enable the user for this group
            ms.member.enable(request.member, memberships=[], notify=False)
        ms.set_state(0)
        ms.save()
    ms.auditLog(f"invite sent by {request.member.username}", "invite", level=21)
    ms.sendInvite(request)
    return ms.restGet(request)


@rd.url(r'^membership$')
@rd.url(r'^membership/(?P<pk>\d+)$')
@rd.login_required
def rest_on_membership(request, pk=None):
    return Membership.on_rest_request(request, pk)


@rd.url(r'^membership/group/(?P<group_id>\d+)$')
@rd.url(r'^membership/me-(?P<group_id>\d+)$')
@rd.login_required
def rest_on_users_membership(request, group_id=None):
    ms = request.member.getMembershipFor(group_id, include_parents=True)
    if ms is None:
        return rv.restStatus(request, False, error="not found", error_code=404)
    if not ms.is_enabled:
        return rv.restPermissionDenied(
            request,
            error=f"membership({request.member.username}) has been disabled for this group({group_id})",
            error_code=410)
    return ms.restGet(request)


@rd.url('group/feed')
@rd.url('group/feed/<int:pk>')
@rd.login_required
def rest_on_group_feed(request, pk=None):
    return GroupFeed.on_rest_request(request, pk)


@rd.urlGET('member/feed')
@rd.urlGET('member/feed/<int:pk>')
@rd.login_required
def rest_on_member_feed(request, pk=None):
    return MemberFeed.on_rest_request(request, pk)


@rd.urlGET('group/stats/kinds')
@rd.login_required
def rest_on_group_stats(request):
    params = request.DATA.asDict()
    if not params.group:
        if not request.member.hasPerm(["view_all_groups", "manage_groups"]):
            return rv.restPermissionDenied()
        out = rh.countOccurences(Group.objects.filter(is_active=True), "kind")
        return rv.restResult(request, dict(data=out))
    # local
    group = Group.objects.filter(pk=params.group).first()
    if not group:
        return rv.restPermissionDenied(request)
    if not request.member.hasPerm(["view_all_groups", "manage_groups"]):
        if not request.member.hasGroupPerm(group, ["reports", "reporting"]):
            return rv.restPermissionDenied(request)
    qset = group.getAllChildren(True, True, True).filter(is_active=True)
    out = rh.countOccurences(qset, "kind")
    return rv.restResult(request, dict(data=out))
    



# from .models import *
# from .forms import *

# from django.shortcuts import get_object_or_404, Http404
# from account.models import User
# from django.db.models import Q

# from rest.views import *
# from rest.decorators import *
# from rest import helpers

# from django.conf import settings

# def medialib_setting(view_func, var, dflt=None):
#     def _wrapper_en(request, *args, **kwargs):
#         return view_func(request, *args, **kwargs)
#     def _wrapper_dis(request, *args, **kwargs):
#         u = getattr(request, 'user')
#         if not u.is_staff:
#             raise Http404
#         return view_func(request, *args, **kwargs)

#     if getattr(settings, var, dflt):
#         return _wrapper_en
#     else:
#         return _wrapper_dis

# def medialib_manage(view_func):
#     return medialib_setting(view_func, 'MEDIALIB_MANAGE_ENABLE')
# def medialib_edit(view_func):
#     return medialib_setting(view_func, 'MEDIALIB_EDIT_ENABLE')

# @urlGET (r'^library/$')
# @vary_on_cookie
# @cache_control(private=True, max_age=300)
# def mediaLibraryList(request):
#     """
#     | Parameters:
#     |	owner: <me|id|username>

#     | Return: REST list of all MediaLibrary data

#     | List all media libraries the user can use
#     """
#     if 'owner' in request.DATA:
#         ids = []
#         for u in request.DATA['owner'].split(","):
#             try:
#                 if u == "me" and request.user and request.user.is_authenticated:
#                     ids.append(str(request.user.id))
#                 elif u.isdigit():
#                     ids.append(str(User.objects.get(id=u).id))
#                 else:
#                     ids.append(str(User.objects.get(username=u).id))
#             except User.DoesNotExist:
#                 raise InternalError("No such user: %s" % u)
#         ret = MediaLibrary.objects.filter(owner__in=ids)
#     elif request.user.has_perm('medialib.can_manage'):
#         ret = MediaLibrary.objects.all()
#     else:
#         ret = MediaLibrary.listPermitted("L", request.user)

#     if 'kind' in request.DATA:
#         ret = ret.filter(Q(allowed_kinds__isnull = True) | Q(allowed_kinds__contains = request.DATA['kind']))

#     return restList(request, ret, exclude=(
#         'allowed_kinds',
#     ), extra=(
#         ('allowed_kinds_list', 'allowed_kinds'),
#     ))


# @urlGET (r'^library/(?P<library_id>\d+)$')
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryGet(request, library_id):
#     """
#     | Get information about specified media library
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("L", request.user)
#     rset = list(lib.rendition_sets.all())
#     defs = RenditionSet.objects.filter(default_set=True)
#     for d in defs:
#         found = False
#         for r in rset:
#             if r.kind == d.kind:
#                 found = True
#                 break
#         if not found:
#             rset.append(d)
#     lib.__rendition_sets = rset
#     return restGet(request, lib, recurse_into=(
#         ('__rendition_sets', 'rendition_sets',),
#     ), exclude=(
#         'rendition_sets.id',
#         'allowed_kinds',
#     ), extra=(
#         ('allowed_kinds_list', 'allowed_kinds'),
#     ))

# @urlGET (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionSetGet(request, library_id, kind):
#     """
#     | Get specifics about a rendition set for a library
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("L", request.user)
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         rset = get_object_or_404(RenditionSet, default_set=True, kind=kind)

#     return restGet(request, rset, recurse_into=(
#         'renditions',
#         ('renditions.preset', ''),
#     ), fields=(
#         'default_set',
#         'kind',
#         'name',
#         'renditions.active',
#         'renditions.id',
#         'renditions.description',
#         'renditions.name',
#         'renditions.use',
#         ('renditions.preset.name', 'preset_name',),
#     ))

# def _copyRenditionSet(old, into, lib):
#     """
#     | make a copy of a rendition set (useful for copying default for customization)
#     """
#     if into:
#         rset = into
#     else:
#         rset = RenditionSet(kind=old.kind, default_set=False, name=old.name)
#     if not rset.id:
#         rset.save()
#     for r in old.renditions.all():
#         rn = RenditionDefinition(name=r.name, description=r.description, preset=r.preset, active=r.active, use=r.use)
#         rn.save()
#         rset.renditions.add(rn)
#         for p in RenditionDefinitionParameterSetting.objects.filter(renditionDefinition=r):
#             pn = RenditionDefinitionParameterSetting(renditionDefinition=rn, parameter=p.parameter, setting=p.setting)
#             pn.save()
#     lib.rendition_sets.add(rset)
#     return rset

# def _getRendition(rset, dflt, kind, rendition_id):
#     try:
#         return rset.renditions.get(pk = rendition_id)
#     except RenditionDefinition.DoesNotExist:
#         if not dflt:
#             dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         try:
#             rendition = dflt.renditions.get(pk = rendition_id)
#         except RenditionDefinition.DoesNotExist:
#             raise Http404
#         try:
#             return rset.renditions.get(name = rendition.name)
#         except RenditionDefinition.DoesNotExist:
#             raise Http404


# @urlPOST (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionSetChange(request, library_id, kind):
#     """
#     | Update rendition set for a library
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     is_new = False
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         rset = RenditionSet(kind=kind, default_set=False, name="new_rendition_set")
#         is_new = True

#     if rset.default_set and not request.user.is_staff:
#         raise InternalError("Cannot edit default set")

#     ret = restSet(request, rset, form=RenditionSetForm)
#     if is_new and rset.id:
#         _copyRenditionSet(dflt, rset, lib)
#     return ret

# @urlPOST (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/reset$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionSetReset(request, library_id, kind):
#     """
#     | reset rendition set to default
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)

#     for rs in lib.rendition_sets.filter(kind=kind):
#         if rs.default_set and not request.user.is_staff:
#             raise InternalError("Cannot edit default set")
#         rs.renditions.all().delete()
#     lib.rendition_sets.filter(kind=kind).delete()
#     return restStatus(request, True)

# @urlPOST (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/clear$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionSetClear(request, library_id, kind):
#     """
#     | clear all renditions from rendition set
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         rset = RenditionSet(kind=kind, default_set=False, name=dflt.name)
#         rset.save()
#         lib.rendition_sets.add(rset)

#     if rset.default_set and not request.user.is_staff:
#         raise InternalError("Cannot edit default set")

#     rset.renditions.all().delete()

#     return restStatus(request, True)

# @urlPOST (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/add$')
# @urlPOST (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/(?P<rendition_id>\d+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionEdit(request, library_id, kind, rendition_id=None):
#     """
#     | add new or edit rendition
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     dflt = None
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         rset = _copyRenditionSet(dflt, None, lib)

#     if rset.default_set and not request.user.is_staff:
#         raise InternalError("Cannot edit default set")

#     if rendition_id:
#         rendition = _getRendition(rset, dflt, kind, rendition_id)
#     else:
#         rendition = None

#     (ret, nrendition) = restSet(request, rendition, form=RenditionDefinitionForm, return_object=True)

#     if not rendition:
#         rset.renditions.add(nrendition)

#     return ret

# @urlDELETE (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/(?P<rendition_id>\d+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionDelete(request, library_id, kind, rendition_id):
#     """
#     | delete rendition
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     dflt = None
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         rset = _copyRenditionSet(dflt, None, lib)

#     if rset.default_set and not request.user.is_staff:
#         raise InternalError("Cannot edit default set")

#     rendition = _getRendition(rset, dflt, kind, rendition_id)
#     rendition.delete()

#     return restStatus(request, True)

# @urlGET (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/(?P<rendition_id>\d+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionGet(request, library_id, kind, rendition_id):
#     """
#     | Get rendition setttings
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         rset = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#     rendition = _getRendition(rset, None, kind, rendition_id)

#     params = {}
#     for p in rendition.preset.configurable_parameters.all():
#         params[p.name] = (p, None, True)
#     rp = rendition.getParameters()
#     for name in rp:
#         if name in params:
#             params[name] = (rp[name][0], rp[name][1], rp[name][2])
#     rendition.__config = list(RenditionDefinitionParameterSetting(parameter=p, setting=v, default=d) for (k, (p, v, d)) in sorted(params.items()))

#     return restGet(request, rendition, recurse_into=(
#         ('preset', ''),
#         ('__config', 'config',),
#         ('config.parameter', '',),
#         'configSet',
#     ), fields=(
#         'active',
#         'id',
#         'description',
#         'name',
#         'use',
#         ('preset.name', 'preset_name',),
#         'config.setting',
#         'config.default',
#         'config.parameter.name',
#         'config.parameter.required',
#         'config.parameter.kind',
#         'config.parameter.description',
#         'config.parameter.choices',
#         'configSet.id',
#         'configSet.name',
#     ))

# @urlGET (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/(?P<rendition_id>\d+)/(?P<config_name>\w+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionGetSetting(request, library_id, kind, rendition_id, config_name):
#     """
#     | Get rendition setttings
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         rset = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#     rendition = _getRendition(rset, None, kind, rendition_id)

#     p = get_object_or_404(RenditionParameter, name=config_name)
#     p.__setting = rendition.getParameter(p)

#     return restGet(request, p, fields=(
#         ('__setting', 'setting',),
#         'name',
#         'required',
#         'kind',
#         'description',
#         'choices',
#     ))

# @urlPOST (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/(?P<rendition_id>\d+)/(?P<config_name>\w+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionSetSetting(request, library_id, kind, rendition_id, config_name):
#     """
#     | Set rendition setttings
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     dflt = None
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         rset = _copyRenditionSet(dflt, None, lib)
#     rendition = _getRendition(rset, dflt, kind, rendition_id)

#     p = get_object_or_404(RenditionParameter, name=config_name)
#     try:
#         setting = p.validate_setting(request.DATA.get('setting', None))
#     except ValueError as e:
#         return restStatus(request, False, error=e.message, errors={"setting": e.message})

#     (cfg, _) = RenditionDefinitionParameterSetting.objects.get_or_create(renditionDefinition=rendition, parameter=p)
#     cfg.setting = setting
#     cfg.save()
#     return restStatus(request, True)

# @urlPOST (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/(?P<rendition_id>\d+)/(?P<config_name>\w+)/reset$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionResetSetting(request, library_id, kind, rendition_id, config_name):
#     """
#     | Set rendition setttings
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     dflt = None
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         return restStatus(request, True)
#     rendition = _getRendition(rset, dflt, kind, rendition_id)

#     p = get_object_or_404(RenditionParameter, name=config_name)
#     RenditionDefinitionParameterSetting.objects.filter(renditionDefinition=rendition, parameter=p).delete()
#     return restStatus(request, True)

# @urlPOST (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/(?P<rendition_id>\d+)/accountConfig/(?P<cfg_id>\d+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionAddAccountConfig(request, library_id, kind, rendition_id, cfg_id):
#     """
#     | Add account config to media lib rendition def
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     dflt = None
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         return restStatus(request, True)
#     rendition = _getRendition(rset, dflt, kind, rendition_id)

#     cfg = get_object_or_404(AccountConfig, pk=cfg_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         cfg.assertAcl("R", request.user)

#     rendition.configSet.add(cfg);

#     return restStatus(request, True)

# @urlDELETE (r'^library/(?P<library_id>\d+)/renditionset/(?P<kind>.)/(?P<rendition_id>\d+)/accountConfig/(?P<cfg_id>\d+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaLibraryRenditionDeleteAccountConfig(request, library_id, kind, rendition_id, cfg_id):
#     """
#     | Add account config to media lib rendition def
#     """
#     lib = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         lib.assertAcl("A", request.user)
#     dflt = None
#     try:
#         rset = lib.rendition_sets.get(kind=kind)
#     except RenditionSet.DoesNotExist:
#         dflt = get_object_or_404(RenditionSet, default_set=True, kind=kind)
#         return restStatus(request, True)
#     rendition = _getRendition(rset, dflt, kind, rendition_id)

#     cfg = get_object_or_404(AccountConfig, pk=cfg_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         cfg.assertAcl("R", request.user)

#     rendition.configSet.remove(cfg);

#     return restStatus(request, True)


# @urlGET (r'^library/(?P<library_id>\d+)/items$')
# @vary_on_cookie
# @cache_control(private=True, max_age=300)
# def mediaLibraryGetItems(request, library_id):
#     """
#     | Get items inside a media library
#     """
#     ret = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         ret.assertAcl("L", request.user)
#     fields=["id", "kind", "library", "name", "description", "state", "render_error", "created", "owner.username", "owner.id"]
#     ret = ret.items.all()
#     if 'kind' in request.DATA:
#         ret = ret.filter(kind__in = request.DATA['kind'].split(","))
#     return restList(request, ret, fields=fields, recurse_into=["owner"])

# @urlPOST (r'^library/(?P<library_id>\d+)$')
# @medialib_manage
# def mediaLibrarySet(request, library_id):
#     """
#     | Update media library metadata
#     """
#     ret = get_object_or_404(MediaLibrary, pk=library_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         ret.assertAcl("W", request.user)
#     return restSet(request, ret, form=MediaLibraryForm)

# @urlGET (r'^item/(?P<item_id>\d+)/renditions$')
# @urlGET (r'^item/(?P<item_id>\d+)\.(?P<token>[^/]*)/renditions$')
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def mediaItemGetRenditions(request, item_id, token=None):
#     """
#     | Get information about specified media item
#     """
#     item = get_object_or_404(MediaItem, pk=item_id)
#     if not item.check_token(token):
#         if not request.user.has_perm('medialib.can_manage'):
#             item.assertAcl("R", request.user)

#     return _mediaItemGetRenditions(request, item)

# def _mediaItemGetRenditions(request, item, accept_list=None):
#     """
#     | Get information about specified media item
#     """
#     renditions = item.renditions.all()
#     if request.DATA.get('use', None):
#         renditions = renditions.filter(use = request.DATA['use'])
#     for f in ('height', 'width', 'bytes', 'created'):
#         if request.DATA.get(f, None):
#             renditions = renditions.filter(**{ f: request.DATA[f] })
#         elif request.DATA.get(f + "_gt", None):
#             renditions = renditions.filter(**{ f+"__gt": request.DATA[f] })
#         elif request.DATA.get(f + "_lt", None):
#             renditions = renditions.filter(**{ f+"__lt": request.DATA[f] })
#     renditions = renditions.order_by('-created')

#     rseen = []
#     ret = []
#     for r in renditions:
#         if r.rendition_definition_id in rseen:
#             continue
#         rseen.append(r.rendition_definition_id)
#         ret.append(r)

#     def cmp_rendition(x, y):
#         r = cmp(x.use, y.use)
#         if r != 0:
#             return r
#         r = cmp(x.bytes, y.bytes)
#         if r != 0:
#             return r
#         r = cmp(x.rendition_definition_id, y.rendition_definition_id)
#         if r != 0:
#             return r
#         r = cmp(x.pk, y.pk)
#         return r
#     ret.sort(cmp=cmp_rendition)

#     return restList(request, ret, size=0, model=MediaItemRendition, exclude=['mediaitem', 'url', 'rendition_definition'], extra=[('view_url', 'url'), ('rtmp_url', 'rtmp')], accept_list=accept_list);

# @urlGET (r'^item/(?P<item_id>\d+)$')
# @urlGET (r'^item/(?P<item_id>\d+)\.(?P<token>[^/]*)$')
# @vary_on_cookie
# @cache_control(private=True, max_age=300)
# def mediaItemGet(request, item_id, token=None):
#     """
#     | Get information about specified media item
#     """
#     ret = get_object_or_404(MediaItem, pk=item_id)
#     if not ret.check_token(token):
#         if not request.user.has_perm('medialib.can_manage'):
#             ret.assertAcl("R", request.user)
#     opts = {
#         'recurse_into': (('cuepoint', 'cuepoints',),),
#     }
#     if ret.kind == 'L':
#         opts['extra'] = (('live_state_ingesting','stream_ingesting',), )
#     data = restGet(request, ret, exclude=['acl_is_default'], accept_list=['data'], **opts)
#     data['renditions'] = _mediaItemGetRenditions(request, ret, accept_list=['data'])
#     return restReturn(request, data)

# @urlPOST (r'^item/(?P<item_id>\d+)/stop$')
# @vary_on_cookie
# def mediaItemStop(request, item_id):
#     """
#     | stop ingesting
#     """
#     item = get_object_or_404(MediaItem, pk=item_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         item.assertAcl("W", request.user)
#     if not item.live_state_ingesting():
#         return restStatus(request, False, error="Not actively ingesting")
#     item.state = 120
#     item.save()
#     return restStatus(request, True)

# @urlPOST (r'^item/(?P<item_id>\d+)$')
# @medialib_edit
# def mediaItemSet(request, item_id):
#     """
#     | Update information about specified media item
#     """
#     ret = get_object_or_404(MediaItem, pk=item_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         ret.assertAcl("W", request.user)
#     return restSet(request, ret, form=MediaItemCanUploadForm)

# @urlDELETE (r'^item/(?P<item_id>\d+)$')
# @medialib_edit
# def mediaItemDelete(request, item_id):
#     """
#     | Delete specified media library
#     """
#     ret = get_object_or_404(MediaItem, pk=item_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         ret.assertAcl("D", request.user)
#     return restDelete(request, ret)


# @urlPOST (r'^library/_/new$')
# @urlPOST (r'^library/(?P<library_id>\d+)/new$')
# @medialib_edit
# def mediaItemNew(request, library_id=None):
#     """
#     | Upload new media item
#     """
#     ret = None
#     if library_id:
#         ret = get_object_or_404(MediaLibrary, pk=library_id)
#         if not request.user.has_perm('medialib.can_manage'):
#             ret.assertAcl("C", request.user)

#     if not ret:
#         if request.user and request.user.is_authenticated:
#             try:
#                 ret = MediaLibrary.objects.filter(owner=request.user)[0]
#             except IndexError:
#                 pass

#     if not ret:
#         try:
#             ret = MediaLibrary.listPermitted('C', request.user)[0]
#         except IndexError:
#             pass

#     if not ret:
#         ret = MediaLibrary(name='Default Library', owner=request.user)
#         ret.save()

#     return restAdd(request, {}, form=MediaItemUploadForm, data_override={ 'library': ret })

# @url (r'^renditionPreset$')
# @medialib_manage
# def listRenditionPresets(request):
#     """
#     | List available presets
#     """
#     return restList(request, RenditionPreset.objects.all(), fields=(
#         'name',
#         'short_name',
#         'description',
#         'stage',
#         'is_distribution',
#         'configurable_parameters.name',
#         'configurable_parameters.description',
#         'configurable_parameters.required',
#         'configurable_parameters.kind',
#         'configurable_parameters.choices',
#     ), recurse_into=(
#         "configurable_parameters",
#     ))

# @url (r'^renditionPreset/(?P<name>\w+)$')
# @medialib_manage
# def getRenditionPresets(request, name):
#     """
#     | Get presets info
#     """
#     ret = get_object_or_404(MediaLibrary, short_name=name)
#     return restGet(request, ret, fields=(
#         'name',
#         'short_name',
#         'description',
#         'stage',
#         'is_distribution',
#         'configurable_parameters.name',
#         'configurable_parameters.description',
#         'configurable_parameters.required',
#         'configurable_parameters.kind',
#         'configurable_parameters.choices',
#     ), recurse_into=(
#         "configurable_parameters",
#     ))

# @urlGET (r'^accountConfig$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def accountConfigList(request):
#     """
#     | Parameters:
#     |	owner: <me|id|username>

#     | Return: REST list of account configs

#     | List available account config sets
#     """
#     if 'owner' in request.DATA:
#         ids = []
#         for u in request.DATA.get('owner', 'me').split(","):
#             try:
#                 if u == "me" and request.user and request.user.is_authenticated:
#                     ids.append(str(request.user.id))
#                 elif u.isdigit():
#                     ids.append(str(User.objects.get(id=u).id))
#                 else:
#                     ids.append(str(User.objects.get(username=u).id))
#             except User.DoesNotExist:
#                 raise InternalError("No such user: %s" % u)
#         ret = AccountConfig.objects.filter(owner__in = ids)
#     else:
#         ret = AccountConfig.listPermitted("L", request.user)

#     if 'applicable_preset' in request.DATA:
#         try:
#             if request.DATA['applicable_preset']:
#                 preset = RenditionPreset.objects.get(short_name = request.DATA['applicable_preset'])
#             else:
#                 preset = None
#         except RenditionPreset.DoesNotExist:
#             return restStatus(request, False, error="Preset not found: %s" % preset, errors={"applicable_preset": "Preset not found: %s" % preset})
#         ret = ret.filter(applicable_preset = preset)

#     return restList(request, ret, fields=(
#         'id',
#         'name',
#         ('!applicable_preset.name', 'applicable_preset',),
#         ('!owner.username', 'owner',),
#     ))

# @urlPOST (r'^accountConfig/(?P<cfg_id>\d+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def accountConfigEdit(request, cfg_id):
#     """
#     | Parameters:
#     |	name: <string>
#     |	applicable_preset: <string>

#     | add/edit accountConfig
#     """
#     cfg = get_object_or_404(AccountConfig, pk=cfg_id)

#     if 'name' in request.DATA:
#         cfg.name = request.DATA['name']
#     if 'applicable_preset' in request.DATA:
#         preset = request.DATA['applicable_preset']
#         try:
#             cfg.applicable_preset = RenditionPreset.objects.get(short_name = preset)
#         except RenditionPreset.DoesNotExist:
#             return restStatus(request, False, error="Preset not found: %s" % preset, errors={"applicable_preset": "Preset not found: %s" % preset})
#     cfg.save()
#     return restStatus(request, True, id=cfg.id)

# @urlDELETE (r'^accountConfig/(?P<cfg_id>\d+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def accountConfigDelete(request, cfg_id):
#     """
#     | Parameters: NONE

#     | delete accountConfig
#     """
#     cfg = get_object_or_404(AccountConfig, pk=cfg_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         cfg.assertAcl("D", request.user)
#     cfg.delete()
#     return restStatus(request, True)

# @urlGET (r'^accountConfig/(?P<cfg_id>\d+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def accountConfigGet(request, cfg_id):
#     """
#     | Parameters: NONE

#     | Return: account config details

#     | Get account config by id
#     """
#     cfg = get_object_or_404(AccountConfig, pk=cfg_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         cfg.assertAcl("R", request.user)

#     params = {}
#     for p in AccountConfigParameterSetting.objects.filter(accountConfig=cfg):
#         params[p.parameter.name] = (p.parameter, p.setting)
#     cfg.__config = list(RenditionDefinitionParameterSetting(parameter=p, setting=v) for (k, (p, v)) in list(params.items()))

#     return restGet(request, cfg, recurse_into=(
#         ('__config', 'config',),
#         ('config.parameter', '',),
#     ), fields=(
#         'id',
#         'name',
#         ('!owner.username', 'owner',),
#         ('!applicable_preset.name', 'applicable_preset',),
#         'config.setting',
#         'config.parameter.name',
#         'config.parameter.required',
#         'config.parameter.kind',
#         'config.parameter.description',
#         'config.parameter.choices',
#     ))

# @urlGET (r'^accountConfig/(?P<cfg_id>\d+)/(?P<config_name>\w+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def accountConfigParameterGet(request, cfg_id, config_name):
#     """
#     | Parameters: NONE

#     | Return: account config details

#     | Get account config by id
#     """
#     cfg = get_object_or_404(AccountConfig, pk=cfg_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         cfg.assertAcl("R", request.user)
#     ret = get_object_or_404(AccountConfigParameterSetting, accountConfig=cfg, parameter__name=config_name)

#     return restGet(request, ret, recurse_into=(
#         ('parameter', '',),
#     ), fields=(
#         'setting',
#         'parameter.name',
#         'parameter.required',
#         'parameter.kind',
#         'parameter.description',
#         'parameter.choices',
#     ))

# @urlPOST (r'^accountConfig/(?P<cfg_id>\d+)/(?P<config_name>\w+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def accountConfigParameterSet(request, cfg_id, config_name):
#     """
#     | Parameters: NONE

#     | Return: account config details

#     | Get account config by id
#     """
#     cfg = get_object_or_404(AccountConfig, pk=cfg_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         cfg.assertAcl("C", request.user)
#     p = get_object_or_404(RenditionParameter, name=config_name)

#     try:
#         setting = p.validate_setting(request.DATA.get('setting', None))
#     except ValueError as e:
#         return restStatus(request, False, error=e.message, errors={"setting": e.message})

#     (s, _) = AccountConfigParameterSetting.objects.get_or_create(accountConfig=cfg, parameter=p)
#     s.setting = setting
#     s.save()
#     return restStatus(request, True)

# @urlDELETE (r'^accountConfig/(?P<cfg_id>\d+)/(?P<config_name>\w+)$')
# @medialib_manage
# @vary_on_cookie
# @cache_control(private=True, max_age=1)
# def accountConfigParameterDel(request, cfg_id, config_name):
#     """
#     | Parameters: NONE

#     | Return: account config details

#     | Get account config by id
#     """
#     cfg = get_object_or_404(AccountConfig, pk=cfg_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         cfg.assertAcl("C", request.user)
#     p = get_object_or_404(RenditionParameter, name=config_name)

#     try:
#         setting = p.validate_setting(request.DATA.get('setting', None))
#     except ValueError as e:
#         return restStatus(request, False, error=e.message, errors={"setting": e.message})

#     s = get_object_or_404(AccountConfigParameterSetting, accountConfig=cfg, parameter=p)
#     s.delete()
#     return restStatus(request, True)

# @urlGET (r'^item/(?P<item_id>\d+)/cue$')
# def mediaItemCueList(request, item_id):
#     ret = get_object_or_404(MediaItem, pk=item_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         ret.assertAcl("R", request.user)

#     return restList(request, ret.cuepoint_set.all())

# @urlPOST (r'^item/(?P<item_id>\d+)/cue$')
# @urlPOST (r'^item/(?P<item_id>\d+)/cue/(?P<cue_id>\d+)$')
# @medialib_edit
# def mediaItemCueSet(request, item_id, cue_id=None):
#     """
#     | Update information about specified media item
#     """
#     item = get_object_or_404(MediaItem, pk=item_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         item.assertAcl("W", request.user)
#     if not item.kind in ('V', 'L', 'A'):
#         return restStatus(request, False, error="Can only cue video assets")

#     if cue_id:
#         cue = get_object_or_404(CuePoint, pk=item_id, item=item)
#     else:
#         cue = None

#     return restSet(request, cue, form=CuePointForm, formopts={"item": item})

# @urlDELETE (r'^item/(?P<item_id>\d+)/cue/(?P<cue_id>\d+)$')
# @medialib_edit
# def mediaItemCueDel(request, item_id, cue_id):
#     """
#     | Update information about specified media item
#     """
#     item = get_object_or_404(MediaItem, pk=item_id)
#     if not request.user.has_perm('medialib.can_manage'):
#         item.assertAcl("W", request.user)
#     if not item.kind in ('V', 'L', 'A'):
#         return restStatus(request, False, error="Can only cue video assets")

#     cue = get_object_or_404(CuePoint, pk=item_id, item=item)

#     return restDelete(request, cue)

# @urlGET (r'^item/search$')
# def mediaItemSearchl(request):
#     """
#     | Parameters:
#     |	q: string
#     """
#     if request.user.has_perm('medialib.can_manage'):
#         ret = MediaItem.objects.filter(state=200)
#     else:
#         ret = MediaItem.listPermitted('L', request.user).filter(state=200)

#     for q in request.DATA.get('q', '').split(' '):
#         ret = ret.filter(Q(name__icontains=q) | Q(library__name__icontains=q) | Q(owner__username__icontains=q) | Q(owner__first_name__icontains=q) | Q(owner__last_name__icontains=q))

#     return restList(request, ret,
#         fields=(
#             "id",
#             "kind",
#             "library",
#             "name",
#             "description",
#             "state",
#             "created",
#             "owner.username",
#             "owner.id",
#             ("!thumbnail_large.view_url", "thumbnail"),
#             ("!image_small.view_url", "image"),
#             ("!thumbnail_animated.view_url", "thumbnail_animated"),
#             ("still.view_url_nonexpire", "still"),
#             ("still.width", "width"),
#             ("still.height", "height"),
#         ),
#         recurse_into=(
#             "owner",
#             ("still",''),
#         ),
#     )

# def saveMediaFile(file, name, file_name=None, is_base64=False):
#     """
#     Generic method to save a media file
#     """
#     if file_name is None:
#         file_name = name
#     # make sure we set the name base64_data
#     if is_base64:
#         mi = MediaItem(name=file_name, base64_data=file)
#     elif type(file) in [str, str] and (file.startswith("https:") or file.startswith("http:")):
#         mi = MediaItem(name=name, downloadurl=file)
#     else:
#         mi = MediaItem(name=name, newfile=file)
#     mi.save()
#     return mi

# from . import ocr
# from shutil import copyfile
# @urlPOST(r'^ocr$')
# @urlPOST(r'^ocr/$')
# def ocr_handler(request):
#     if not ocr.hasTesseract():
#         return restPermissionDenied(request, error="tesseract not installed")
#     b64 = request.DATA.get("image")
#     if not b64 and len(request.FILES) == 0:
#         return restPermissionDenied(request, error="no files found")
#     if b64:
#         # print(b64)
#         # print(b64[:64].decode("base64"))
#         img_data = b64.decode('base64')
#         print(("img size: {}".format(len(b64))))
#     else:
#         img_filename = list(request.FILES.keys())[0]
#         img_file = request.FILES[img_filename]
#         img_file.seek(0)
#         img_data = img_file.read()

#     output = ""
#     charset = request.DATA.get("charset", None)
#     pp_dpi = request.DATA.get("pp_dpi", field_type=bool)
#     pp_noise = request.DATA.get("pp_noise", field_type=bool)

#     with helpers.TemporaryFile() as tmp_path:
#         charset = request.DATA.get("charset", None)
#         with open(tmp_path, 'w') as temp:
#             temp.write(img_data)
#             temp.flush()
#         # copyfile(tmp_path, "..../var/ocr.png")
#         output = ocr.ocrImage(tmp_path, pp_dpi=pp_dpi, pp_noise=pp_noise, charset=charset)
#         # copyfile(tmp_path, "..../var/ocr2.png")
#     print("ocr output:")
#     print(output)
#     return restGet(request, {"text":output})



# # BEGIN NEW REST
# @url(r'^media/item/$')
# @url(r'^media/item/(?P<pk>\d+)$')
# @login_required
# def media_item_action(request, pk=None):
#     return MediaItem.on_rest_request(request, pk)

# @url(r'^media/item/ref$')
# @url(r'^media/item/ref/(?P<pk>\d+)$')
# @login_required
# def media_item_action(request, pk=None):
#     return MediaItemRef.on_rest_request(request, pk)





# Generated by Django 4.1.3 on 2022-12-20 04:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('metrics', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='metrics',
            name='k11',
            field=models.SlugField(default=None, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='metrics',
            name='k12',
            field=models.SlugField(default=None, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='metrics',
            name='k13',
            field=models.SlugField(default=None, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='metrics',
            name='k14',
            field=models.SlugField(default=None, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='metrics',
            name='v11',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='metrics',
            name='v12',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='metrics',
            name='v13',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='metrics',
            name='v14',
            field=models.IntegerField(default=0),
        ),
    ]



import time

from rest import UberDict

from ws4redis.redis import RedisMessage, RedisStore, getRedisClient, getPoolStatus


def buildEventMessage(name=None, message=None, priority=0, model=None, model_pk=None, custom=None):
    msg = UberDict(priority=priority)
    if name:
        msg["name"] = name

    if message:
        msg["message"] = message

    if model:
        msg["component"] = UberDict(pk=model_pk, model=model)

    if custom:
        msg.update(custom)
    return msg.toJSON(as_string=True)


def ping():
    return getRedisClient().ping()


def exists(key, default=None):
    c = getRedisClient()
    return c.exists(key)


def keys(keys):
    c = getRedisClient()
    return [v.decode() for v in c.keys(keys)]


def get(key, default=None, field_type=None):
    c = getRedisClient()
    v = c.get(key)
    if v is None:
        return default
    if field_type is not None:
        if field_type == "json":
            return UberDict.fromJSON(v, ignore_errors=True)
        if field_type in [str, "str"]:
            v = v.decode()
        return field_type(v)
    return v


def set(key, value, expire=None):
    c = getRedisClient()
    v = c.set(key, value)
    if expire:
        c.expire(key, expire)
    return v


def incr(key, amount=1, expire=None):
    c = getRedisClient()
    v = c.incr(key, amount)
    if expire:
        c.expire(key, expire)
    return v


def expire(key, expire):
    c = getRedisClient()
    return c.expire(key, expire)


def decr(key, amount=1):
    c = getRedisClient()
    return c.decr(key, amount)


def delete(key):
    c = getRedisClient()
    return c.delete(key)


# SET FUNCTIONS
def sadd(name, *values):
    # add value to set
    c = getRedisClient()
    return c.sadd(name, *values)


def srem(name, *values):
    # remove value from set
    c = getRedisClient()
    return c.srem(name, *values)


def sismember(name, value):
    # return items in set
    c = getRedisClient()
    return c.sismember(name, value)


def scard(name):
    # count items in set
    c = getRedisClient()
    return c.scard(name)


def smembers(name):
    # return items in set
    c = getRedisClient()
    return c.smembers(name)


# HASH FUNCTIONS
def hget(name, field, default=None):
    c = getRedisClient()
    v = c.hget(name, field)
    if v is None:
        return default
    return v


def hgetall(name):
    c = getRedisClient()
    return c.hgetall(name)


def hset(name, field, value):
    c = getRedisClient()
    return c.hset(name, field, value)


def hdel(name, field):
    c = getRedisClient()
    return c.hdel(name, field)


def hincrby(name, field, inc=1):
    c = getRedisClient()
    return c.hincrby(name, field, inc)


def lpush(name, value, unique=False):
    c = getRedisClient()
    if isinstance(value, list):
        for v in value:
            if unique and value.encode() in c.lrange(name, 0, -1):
                return 0
            c.lpush(name, v)
        return len(value)
    if unique and value.encode() in c.lrange(name, 0, -1):
        return 0
    return c.lpush(name, value)


def rpush(name, value, unique=False):
    c = getRedisClient()
    if isinstance(value, list):
        for v in value:
            if unique and value.encode() in c.lrange(name, 0, -1):
                return 0
            c.rpush(name, v)
        return len(value)
    if unique and value.encode() in c.lrange(name, 0, -1):
        return 0
    return c.rpush(name, value)


def lpop(name, timeout=None):
    c = getRedisClient()
    if timeout is None:
        return c.lpop(name)
    r = c.blpop(name, timeout=timeout)
    if isinstance(r, tuple):
        return r[1].decode()


def rpop(name, timeout=None):
    c = getRedisClient()
    if timeout is None:
        return c.rpop(name)
    r = c.brpop(name, timeout=timeout)
    if isinstance(r, tuple):
        return r[1].decode()


def lrem(name, value, occurences=1):
    c = getRedisClient()
    return c.lrem(name, occurences, value.encode())


def vpop(name, value, timeout=None):
    c = getRedisClient()
    start = time.time()
    while c.lrem(name, 1, value) == 0:
        if timeout is None:
            return 0
        time.sleep(1.0)
        elapsed = time.time() - start
        if elapsed > timeout:
            return 0
    return 1


def lrange(name, start, end):
    c = getRedisClient()
    return [v.decode() for v in c.lrange(name, start, end)]


def sendToUser(user, name, message=None, priority=0, model=None, model_pk=None, custom=None):
    return sendMessageToUsers([user], buildEventMessage(name, message, priority, model, model_pk, custom))


def sendToUsers(users, name, message=None, priority=0, model=None, model_pk=None, custom=None):
    return sendMessageToUsers(users, buildEventMessage(name, message, priority, model, model_pk, custom))


def sendMessageToUsers(users, msg):
    return RedisStore().publish(RedisMessage(msg), channel="user", pk=[u.username for u in users])


def sendToGroup(group, name, message=None, priority=0, model=None, model_pk=None, custom=None):
    return sendMessageToModels("group", [group], buildEventMessage(name, message, priority, model, model_pk, custom))


def sendToGroups(groups, name, message=None, priority=0, model=None, model_pk=None, custom=None):
    return sendMessageToModels("group", groups, buildEventMessage(name, message, priority, model, model_pk, custom))


def sendToModels(channel, models, name, message=None, priority=0, model=None, model_pk=None, custom=None):
    return sendMessageToModels(channel, models, buildEventMessage(name, message, priority, model, model_pk, custom))


def sendMessageToModels(channel, models, msg):
    return RedisStore().publish(RedisMessage(msg), channel=channel, pk=[g.pk for g in models])


def sendMessageToPK(channel, pk, msg):
    return RedisStore().publish(RedisMessage(msg), channel=channel, pk=pk)


def broadcast(name, message=None, priority=0, model=None, model_pk=None, custom=None):
    return broadcastMessage(buildEventMessage(name, message, priority, model, model_pk, custom))


def broadcastMessage(msg):
    return RedisStore().publish(RedisMessage(msg), channel="broadcast")


def publish(key, data, c=None):
    if c is None:
        c = getRedisClient()
    if isinstance(data, dict):
        if not isinstance(data, UberDict):
            data = UberDict(data)
        data = data.toJSON(as_string=True)
    return c.publish(key, data)


def subscribe(channel):
    c = getRedisClient()
    pubsub = c.pubsub()
    pubsub.subscribe(channel)
    return pubsub


def waitForMessage(pubsub, msg_filter, timeout=55):
    timeout_at = time.time() + timeout
    while time.time() < timeout_at:
        message = pubsub.get_message()
        if message is not None:
            if message.get("type") == "message":
                msg = UberDict.fromJSON(message.get("data"))
                if msg_filter(msg):
                    pubsub.unsubscribe()
                    return msg
        time.sleep(1.0)
    pubsub.unsubscribe()
    return None


def isOnline(name, pk):
    return sismember(f"{name}:online", pk)



from rest import settings
from rest import ssl_check
from datetime import datetime, timedelta
import incident


def run_monitoring(task):
    checkDomains(task)


def checkDomains(task):
    if not settings.DOMAIN_WATCH:
        task.log("no domains to check")
        return False
    now = datetime.utcnow()
    alarm_date = now + timedelta(days=30)
    result = ssl_check.check(*settings.DOMAIN_WATCH)
    for key, value in result.items():
        if value is None or isinstance(value, str):
            continue
        if value < alarm_date:
            task.log(f"{key} is expiring soon")
            notifyDomainExpiring(key, value)


def notifyDomainExpiring(domain, expires):
    incident.event_now(
        "domain_expires",
        f"{domain} is expiring",
        level=1, hostname=domain,
        domain=domain)




from django.db import models
from rest import models as rm
from rest import fields as rf
from rest import settings
from objict import objict
from datetime import datetime


class UserPassKey(models.Model, rm.RestModel):
    """
    A “Passkey” is a modern, secure authentication method designed to replace
    traditional passwords with more robust and user-friendly alternatives.
    It leverages cryptographic techniques and is based on standards like
    WebAuthn and FIDO2, aiming to improve both security and convenience.
    """
    class RestMeta:
        VIEW_PERMS = ["manage_users", "owner"]
        CAN_DELETE = True
        VIEW_PERMS_MEMBER_FIELD = "member"
        GRAPHS = {
            "default": {
                "fields": [
                    "id", "created", "modified", "last_used",
                    "uuid", "rp_id", "is_enabled", 
                    "name", "platform", "info"
                ],
            }
        }
        
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    member = models.ForeignKey("account.Member", related_name="passkeys", on_delete=models.CASCADE)

    uuid = models.CharField(max_length=255, unique=True)
    rp_id = models.CharField(max_length=255, null=False, db_index=True)
    is_enabled = models.BooleanField(default=True, db_index=True)

    name = models.CharField(max_length=255)
    platform = models.CharField(max_length=255, default='')
    last_used = models.DateTimeField(null=True, default=None)
    token = models.CharField(max_length=255, null=False)

    info = rf.JSONField()

    def touch(self):
        self.last_used = datetime.now()
        self.save()






from .location import GeoLocation  # noqa: F401
from .ip import GeoIP  # noqa: F401
from .address import Address  # noqa: F401
from .track import GeoPosition, GeoTrack  # noqa: F401
from .legacy import GeoIPLocation  # noqa: F401



from django.test import TestCase
from django.test import Client
from .models import *
from django.db.models import Count
import json
from test_extensions.django_common import DjangoCommon

from account.tests import RPC_LoginHelper
class RPC_MediaLibTest(DjangoCommon,RPC_LoginHelper):
    fixtures = ['medialib_test_fixture']
    
    def setUp(self):
        self.login_helper_setUp()
    def test_mediaLibraryList(self):
        logged_client = self.default_client_login_helper()
        url = "/medialib/library/"
        params = {}
        response = logged_client.get(url, params)
        data = json.loads(response.content)
        returned = data['data'][0]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(returned["owner"] , 2)
        self.assertEqual(returned["acl_is_default"] , True)
        self.assertEqual(returned["name"] , "User Media Library")
    
    
    def test_mediaLibraryGetItems(self):
        logged_client = self.default_client_login_helper()
        url = "/medialib/library/6/items"  #library id is 6 to match data on the test fixture
        params = {}
        response = logged_client.get(url, params)
        # return the items from a specific media library
        self.assertEqual(response.status_code, 200) # no crash

        data = json.loads(response.content)
        returned = data['data'][0]
        self.assertEqual(returned["kind"] , "I")
        self.assertEqual(returned["description"] , "test")
        self.assertEqual(returned["render_error"] , "")
        self.assertEqual(returned["name"] , "test")

            
    def test_mediaLibraryGet(self):
        logged_client = self.default_client_login_helper()
        url = "/medialib/library/6"
        params = {}
        response = logged_client.get(url, params)
        self.assertEqual(response.status_code, 200) # no crash
        self.assert_response_contains('User Media Library',response)
        #Now test for invalid object.
        url = "/medialib/library/100"
        params = {}
        response = logged_client.get(url, params)
        self.assertEqual(response.status_code, 404) # no crash

    def NOtest_mediaLibrarySet(self):
        logged_client = self.default_client_login_helper()
        url = "/medialib/library/6"
        params = {'description':'changed description'}
        response = logged_client.post(url, params)
        self.assertEqual(response.status_code, 200) # no crash
        # verify description field is changed
        desc = MediaLibrary.objects.get(pk=6).description
        self.assertEqual(desc,'changed description')


    def NOtest_mediaItemSet(self):
        logged_client = self.default_client_login_helper()
        url = "/medialib/item/2" # item is 2 to fit fixture
        params = {'description':'changed description'}
        response = logged_client.post(url, params)
        self.assertEqual(response.status_code, 200) # no crash
        # verify description field is changed
        desc = MediaItem.objects.get(pk=2).description
        self.assertEqual(desc,'changed description')


    def NOtest_mediaItemNew(self):
        #create a fake .gif image file for testing
        import base64
        import io
        gif64 = "R0lGODdhAQABAIABAGtdWv///ywAAAAAAQABAAACAkQBADs="
        mygif =  base64.b64decode(gif64)
        fake_file = io.StringIO(mygif)
        fake_file.name = 'noimage.gif'
        #now upload the gif using the API
        libcount= MediaItem.objects.all().count()
        logged_client = self.default_client_login_helper()
        url = "/medialib/library/6/new"
        params = {'name':'noimage.gif','file':fake_file}
        response = logged_client.post(url, params)
        self.assertEqual(response.status_code, 200) # no crash
        data = json.loads(response.content)
        self.assertEqual(data['status'],True)
        self.assertEqual(libcount+1,MediaItem.objects.all().count())
    
    def test_mediaItemGet(self):
        logged_client = self.default_client_login_helper()
        url = "/medialib/item/2"
        params = {}
        response = logged_client.get(url, params)
        data = json.loads(response.content)
        self.assertEqual(response.status_code, 200) # no crash
        expected  = {"kind": "I",
                     "description": "test",
                     "created": 1314740695.0,
                     "library": 6,
                     "render_error": "",
                     "state": 200,
                     "id": 2,
                     "name": "test"}
        self.assertEqual(response.status_code, 200) # no crash
        for key,value in list(expected.items()) :
            self.assertEqual(data[key], value,key)  # data matches fixture


    def test_mediaItemGet_Token(self):
        VALID_TOKEN = '2.gv.cdc4ec2fa3e4a05a1e65' #2. is the item id
        logged_client = self.default_client_login_helper()
        url = "/medialib/item/"+VALID_TOKEN+"/renditions"
        params = {}
        response = logged_client.get(url, params)
        data = json.loads(response.content)
        self.assertEqual(response.status_code, 200) # no crash
        expected  = {'use': 'video',
                     'name': '360p',
                     'created': 1314813828.0,
                     'bytes': 719486,
                     'height': 360,
                     'width': 640,
                     'id': 1
                     }
        self.assertEqual(response.status_code, 200) # no crash
        for key,value in list(expected.items()) :
            self.assertEqual(data['data'][0][key], value,key)  # data matches fixture

    
    def NOtest_mediaItemDelete(self):
        logged_client = self.default_client_login_helper()
        url = "/medialib/item/2"
        params = {}
        self.assertEqual(1,MediaItem.objects.filter(pk=2).count()) #verify item is there
        response = logged_client.delete(url, params)
        self.assertEqual(response.status_code, 200) # no crash
        self.assertEqual(0,MediaItem.objects.filter(pk=2).count()) #test item is NOT there




# from pillow import Image
from PIL import Image
Image.ANTIALIAS = Image.LANCZOS  # hotfix for deprecation
from PIL import ImageFilter, ImageOps, ImageEnhance

try:
    import cv2
except:
    cv2 = None

try:
    import numpy as np
except:
    np = None

from rest import helpers

def set_image_dpi(input_path, output_path):
    im = Image.open(input_path)
    length_x, width_y = im.size
    factor = min(1, float(1024.0 / length_x))
    size = int(factor * length_x), int(factor * width_y)
    print(("old size: {}".format(im.size)))
    print(("new size: {}".format(size)))
    im_resized = im.resize(size, Image.ANTIALIAS)
    im_resized.save(output_path, format="PNG", dpi=(300, 300))
    return output_path

def set_image_contrast(input_path, output_path):
    img = Image.open(input_path)
    scale_value=scale1.get()
    img = ImageEnhance.Contrast(img).enhance(scale_value)
    img.save(output_path, format="PNG", dpi=(300, 300))
    return output_path

def preproc_image(image_path):
    img = Image.open(image_path)
    img = img.convert('L')
    img = ImageOps.grayscale(img)
    # im = im.filter(ImageFilter.GaussianBlur)
    w, h = img.size
    factor = 4
    nw = w * factor
    nh = h * factor
    # scale_value=scale1.get()
    # img = ImageEnhance.Color(img).enhance(0)
    img = ImageEnhance.Brightness(img).enhance(2)
    # img = ImageEnhance.Sharpness(img).enhance(2)
    img = ImageEnhance.Contrast(img).enhance(4)
    img = img.filter(ImageFilter.GaussianBlur(1.1))
    img = ImageEnhance.Sharpness(img).enhance(6)

    img_resized = img.resize((nw, nh), Image.ANTIALIAS)
    img_resized.save(image_path, format="PNG", dpi=(300, 300))
    # return preproc_cv(image_path)
    return image_path

def preproc_cv(image_path):
    img = cv2.imread(image_path, 0)
    # Convert to gray    
    # img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)    # Apply dilation and erosion to remove some noise    
    # kernel = np.ones((1, 1), np.uint8)    
    # img = cv2.dilate(img, kernel, iterations=1)    
    # img = cv2.erode(img, kernel, iterations=1)
    img = cv2.medianBlur(img, 3)
    # img = cv2.bilateralFilter(img,9,75,75)
    # img = cv.bilateralFilter(img,9,75,75)
    output_file = image_path + ".png"
    cv2.imwrite(output_file, img)
    os.rename(output_file, image_path)
    return image_path

def preproc_cv2(image_path):
    img = cv2.imread(image_path, 0)
    ret,thresh_img = cv2.threshold(img, 0, 255, cv2.THRESH_BINARY_INV|cv2.THRESH_OTSU)
    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (4,8))
    # morph_img = cv2.morphologyEx(thresh_img, cv2.MORPH_CLOSE, kernel)

    opening = cv2.morphologyEx(thresh_img, cv2.MORPH_OPEN, kernel)
    closing = cv2.morphologyEx(opening, cv2.MORPH_CLOSE, kernel)
    # img = image_smoothening(img)
    or_image = cv2.bitwise_or(img, closing)

    output_file = image_path + ".png"
    cv2.imwrite(output_file, or_image)
    os.rename(output_file, image_path)
    return image_path

def preprocess_image_using_pil(image_path):
    # unblur, sharpen filters
    img = Image.open(image_path)
    img = img.convert("RGBA")

    pixdata = img.load()

    # Make the letters bolder for easier recognition
    
    for y in range(img.size[1]):
        for x in range(img.size[0]):
            if pixdata[x, y][0] < 90:
                pixdata[x, y] = (0, 0, 0, 255)

    for y in range(img.size[1]):
        for x in range(img.size[0]):
            if pixdata[x, y][1] < 136:
                pixdata[x, y] = (0, 0, 0, 255)

    for y in range(img.size[1]):
        for x in range(img.size[0]):
            if pixdata[x, y][2] > 0:
                pixdata[x, y] = (255, 255, 255, 255)

    # And sharpen it
    img.filter(ImageFilter.SHARPEN)
    #   Make the image bigger (needed for OCR)
    basewidth = 1000  # in pixels
    im_orig = img
    wpercent = (basewidth/float(im_orig.size[0]))
    hsize = int((float(im_orig.size[1])*float(wpercent)))
    big = img.resize((basewidth, hsize), Image.ANTIALIAS)

    # tesseract-ocr only works with TIF so save the bigger image in that format
    big.save(image_path, format="PNG", dpi=(300,300))
    return image_path

def is_cv2(or_better=False):
    # grab the OpenCV major version number
    major = int(cv2.__version__.split(".")[0])
    # check to see if we are using *at least* OpenCV 2
    if or_better:
        return major >= 2
    # otherwise we want to check for *strictly* OpenCV 2
    return major == 255

def detect_and_rotate(image):
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    gray = cv2.GaussianBlur(gray, (5, 5), 0)
    edged = cv2.Canny(gray, 10, 50)
    cnts = cv2.findContours(edged.copy(), cv2.RETR_LIST, cv2.CHAIN_APPROX_SIMPLE)
    cnts = cnts[0] if is_cv2() else cnts[1]
    cnts = sorted(cnts, key=cv2.contourArea, reverse=True)[:5]
    screenCnt = None
    for c in cnts:
        peri = cv2.arcLength(c, True)
        approx = cv2.approxPolyDP(c, 0.02 * peri, True)
        if len(approx) == 4:
            screenCnt = approx
            break
    cv2.drawContours(image, [screenCnt], -1, (0, 255, 0), 2)
    pts = np.array(screenCnt.reshape(4, 2) * ratio)
    return four_point_transform(orig, pts)


def order_points(pts):
    # initialzie a list of coordinates that will be ordered
    # such that the first entry in the list is the top-left,
    # the second entry is the top-right, the third is the
    # bottom-right, and the fourth is the bottom-left
    rect = np.zeros((4, 2), dtype="float32")

    # the top-left point will have the smallest sum, whereas
    # the bottom-right point will have the largest sum
    s = pts.sum(axis=1)
    rect[0] = pts[np.argmin(s)]
    rect[2] = pts[np.argmax(s)]

    # now, compute the difference between the points, the
    # top-right point will have the smallest difference,
    # whereas the bottom-left will have the largest difference
    diff = np.diff(pts, axis=1)
    rect[1] = pts[np.argmin(diff)]
    rect[3] = pts[np.argmax(diff)]

    # return the ordered coordinates
    return rect

def four_point_transform(image, pts):
    # obtain a consistent order of the points and unpack them
    # individually
    rect = order_points(pts)
    (tl, tr, br, bl) = rect

    # compute the width of the new image, which will be the
    # maximum distance between bottom-right and bottom-left
    # x-coordiates or the top-right and top-left x-coordinates
    widthA = np.sqrt(((br[0] - bl[0]) ** 2) + ((br[1] - bl[1]) ** 2))
    widthB = np.sqrt(((tr[0] - tl[0]) ** 2) + ((tr[1] - tl[1]) ** 2))
    maxWidth = max(int(widthA), int(widthB))

    # compute the height of the new image, which will be the
    # maximum distance between the top-right and bottom-right
    # y-coordinates or the top-left and bottom-left y-coordinates
    heightA = np.sqrt(((tr[0] - br[0]) ** 2) + ((tr[1] - br[1]) ** 2))
    heightB = np.sqrt(((tl[0] - bl[0]) ** 2) + ((tl[1] - bl[1]) ** 2))
    maxHeight = max(int(heightA), int(heightB))

    # now that we have the dimensions of the new image, construct
    # the set of destination points to obtain a "birds eye view",
    # (i.e. top-down view) of the image, again specifying points
    # in the top-left, top-right, bottom-right, and bottom-left
    # order
    dst = np.array([
        [0, 0],
        [maxWidth - 1, 0],
        [maxWidth - 1, maxHeight - 1],
        [0, maxHeight - 1]], dtype="float32")

    # compute the perspective transform matrix and then apply it
    M = cv2.getPerspectiveTransform(rect, dst)
    warped = cv2.warpPerspective(image, M, (maxWidth, maxHeight))
    return warped


def remove_noise_and_smooth1(file_name, output_file=None):
    im = Image.open(file_name)
    white = im.filter(ImageFilter.BLUR).filter(ImageFilter.MaxFilter(15))
    grey = im.convert('L')
    width,height = im.size
    impix = im.load()
    whitepix = white.load()
    greypix = grey.load()
    # for y in range(height):
    #     for x in range(width):
    #         greypix[x,y] = min(255, max(255 * impix[x,y][0] / whitepix[x,y][0], 255 * impix[x,y][2] / whitepix[x,y][3], 255 * impix[x,y][4] / whitepix[x,y][5]))
    for y in range(height):
        for x in range(width):
            greypix[x,y] = min(255, max(255 + impix[x,y][0] - whitepix[x,y][0], 255 + impix[x,y][7] - whitepix[x,y][8], 255 + impix[x,y][9] - whitepix[x,y][10]))
    if output_file:
        greypix.save(output_file, format="PNG", dpi=(300, 300))
    return greypix

def remove_noise_and_smooth(file_name, output_file=None):
    im = Image.open(file_name)
    grey = im.convert('L')
    bw = grey.point(lambda x: 0 if x<150 else 250, '1')
    if output_file:
        bw.save(output_file, format="PNG", dpi=(300, 300))
    return bw

def remove_noise_and_smooth2(file_name, output_file=None):
    # read image as grey scale
    #grey_img = cv2.imread('/home/img/python.png', cv2.IMREAD_GRAYSCALE)
    # save image
    #status = cv2.imwrite('/home/img/python_grey.png',grey_img)
    img = cv2.imread(file_name, 0)
    filtered = cv2.adaptiveThreshold(img.astype(np.uint8), 255, cv2.ADAPTIVE_THRESH_MEAN_C, cv2.THRESH_BINARY, 9, 41)
    kernel = np.ones((1, 1), np.uint8)
    opening = cv2.morphologyEx(filtered, cv2.MORPH_OPEN, kernel)
    closing = cv2.morphologyEx(opening, cv2.MORPH_CLOSE, kernel)
    # img = image_smoothening(img)
    or_image = cv2.bitwise_or(img, closing)
    if output_file:
        cv2.imwrite(output_file, or_image)
    return or_image


def safe_cmd(cmd, *args):
    try:
        cmd_args = [cmd]
        cmd_args.extend(list(args))
        print(cmd_args)
        return subprocess.check_output(cmd_args)
    except subprocess.CalledProcessError as exc:
        return exc.output
    return None



from rest.decorators import periodic
from datetime import datetime, timedelta
from .models import Member, NotificationRecord
from sessionlog.models import SessionLog


@periodic(minute=15)
def run_cleanup_tokens(force=False, verbose=False, now=None):
    # we want to nuke invite tokens every 15 minutes
    # we do not want to do this if using invite
    stale = datetime.now() - timedelta(hours=48)
    qset = Member.objects.filter(auth_code__isnull=False).filter(modified__lte=stale)
    qset.update(auth_code=None)

    # lets prune old non active sessions
    SessionLog.Clean(limit=10000)


@periodic(day=5, hour=11, minute=30)
def run_cleanup_junk(force=False, verbose=False, now=None):
    # cleanup misc data like email notifications
    stale = datetime.now() - timedelta(days=90)
    qset = NotificationRecord.objects.filter(created__lte=stale)
    qset.delete()



from rest import decorators as rd
from rest import views as rv
from medialib import models as medialib


@rd.url(r'media/item/$')
@rd.url(r'media/item/(?P<pk>\d+)$')
@rd.login_required
def rest_on_media_item(request, pk=None):
    return medialib.MediaItem.on_rest_request(request, pk)


@rd.urlPOST('media/item/s3')
@rd.login_required
@rd.requires_params(["filename", "filesize"])
def rest_on_media_upload_s3(request, pk=None):
    return rv.restReturn(request, dict(data=medialib.MediaItem.on_upload_s3(request)))


@rd.url(r'media/ref/$')
@rd.url(r'media/ref/(?P<pk>\d+)$')
@rd.login_required
def rest_on_media_ref(request, pk=None):
    return medialib.MediaItemRef.on_rest_request(request, pk)


@rd.url(r'library/$')
@rd.url(r'library/(?P<pk>\d+)$')
@rd.login_required
def rest_on_library(request, pk=None):
    return medialib.MediaLibrary.on_rest_request(request, pk)
